// Copyright (C) 2011 David Sugar, Haakon Eriksen, GNU Free Call Foundation
//
// This file is part of SwitchView.
//
// SwitchView is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// SwitchView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SwitchView.  If not, see <http://www.gnu.org/licenses/>.

#include "switchview.h"
#include <QTimer>

SwitchView *SwitchView::instance = NULL;
QSystemTrayIcon *SwitchView::trayicon = NULL;

bool testing = false;
bool startup = false;
bool alwaysopen = false;

static QAction *optionsAction;
static QAction *mappedAction;
static QAction *reconnectAction;
static QAction *reloadAction;
static bool connected = false;
static bool stopping = false;
static bool initial = true;

static void warning(void)
{
    if(Mapped::instance->isVisible())
        return;

    QString text = SwitchView::tr("No server connected");
    QMessageBox::warning(SwitchView::instance, SwitchView::tr("SwitchView"), text);
}

SwitchView::SwitchView() :
CoastalMain()
{
    // singleton...
    Q_ASSERT(instance == NULL);
    instance = this;

    QAction *aboutAction = new QAction(tr("About"), this);
    aboutAction->setIcon(QIcon::fromTheme("help-about"));
    aboutAction->setIconVisibleInMenu(true);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));

    optionsAction = new QAction(tr("&Options"), this);
    optionsAction->setIcon(QIcon::fromTheme("preferences-desktop"));
    optionsAction->setIconVisibleInMenu(true);
    connect(optionsAction, SIGNAL(triggered()), this, SLOT(openOptions()));

    reconnectAction = new QAction(tr("Reconnect"), this);
    reconnectAction->setIcon(QIcon::fromTheme("connect_creating"));
    reconnectAction->setIconVisibleInMenu(true);
    connect(reconnectAction, SIGNAL(triggered()), this, SLOT(reconnect()));

    reloadAction = new QAction(tr("Reload"), this);
    reloadAction->setIcon(QIcon::fromTheme("view-refresh"));
    reloadAction->setIconVisibleInMenu(true);

    mappedAction = new QAction(tr("&Status"), this);
    mappedAction->setIcon(QIcon::fromTheme("dialog-information"));
    mappedAction->setIconVisibleInMenu(true);
    connect(mappedAction, SIGNAL(triggered()), this, SLOT(openMapped()));

    QAction *quitAction = new QAction(tr("&Quit"), this);
    quitAction->setIcon(QIcon::fromTheme("application-exit"));
    quitAction->setIconVisibleInMenu(true);
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));

    QApplication::setQuitOnLastWindowClosed(false);

    program_name = "SwitchView";
    program_about = "SIP Witch Desktop GUI Manager";
    setWindowTitle(program_name);
    setWindowIcon(QIcon(":/icons/apps/switchview.png"));
    setWindowFlags(Qt::Window);

    QMenu *traymenu = new QMenu(NULL);
    traymenu->addAction(aboutAction);
    traymenu->addAction(optionsAction);
    traymenu->addSeparator();
    traymenu->addAction(reconnectAction);
    traymenu->addAction(reloadAction);
    traymenu->addAction(mappedAction);
    traymenu->addSeparator();
    traymenu->addAction(quitAction);

    Options::start();

    trayicon = new QSystemTrayIcon(this);
    if(trayicon && Options::isTray()) {
        trayicon->setContextMenu(traymenu);
        trayicon->setIcon(QIcon(":/icons/status/switchview/down.png"));
        trayicon->show();
    }

    connect(qApp, SIGNAL(aboutToQuit()),
        this, SLOT(stop()));

    if(Options::isTray())
        connect(trayicon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
            this, SLOT(action(QSystemTrayIcon::ActivationReason)));

    Events::start();
    Mapped::start();

    if(!Options::isTray())
        QApplication::setQuitOnLastWindowClosed(true);

    if(alwaysopen)
        openMapped();
}

SwitchView::~SwitchView()
{
    Options::instance->setParent(NULL);   // kill connection to mapped before close...
    Mapped::instance->setParent(NULL);

    stopping = true;

    Mapped::stop();
    Events::stop();
    Options::stop();
}

void SwitchView::stop(void)
{
    delete this;
    instance = NULL;
}

void SwitchView::online(void)
{
    if(connected)
        return;

    if(!initial && Options::instance->notify[NOTIFY_CONNECTING])
        notify(tr("SwitchView"), tr("Reconnected to the server"));

    initial = false;

    if(Options::isTray())
        trayicon->setIcon(QIcon(":/icons/status/switchview/live.png"));
#if defined(AF_UNIX) && !defined(_MSWINDOWS_)
    reconnectAction->setVisible(false);
#endif
    mappedAction->setVisible(true);
    connected = true;
}

void SwitchView::reconnect(void)
{
#if defined(AF_UNIX) && !defined(_MSWINDOWS_)
    if(connected)
        return;

#else
    if(connected) {
        Mapped::instance->close();
        mappedAction->setEnabled(false);
        if(Options::isTray())
            trayicon->setIcon(QIcon(":/icons/status/switchview/down.png"));
        connected = false;
    }
#endif
    Events::reconnect();
}

void SwitchView::offline(void)
{
    if(initial) {
        if(!startup)
            warning();
        initial = false;
        return;
    }

    if(!connected)
        return;

    Mapped::instance->close();

    if(Options::instance->notify[NOTIFY_DISCONNECT])
        notify(tr("SwitchView"), tr("Server connection lost"), QSystemTrayIcon::Critical);
    else if(!alwaysopen)
        QMessageBox::critical(NULL, tr("SwitchView"),
            tr("Server connection lost\n\n"));

    if(Options::isTray())
        trayicon->setIcon(QIcon(":/icons/status/switchview/down.png"));
    reconnectAction->setVisible(true);
    connected = false;
}

void SwitchView::openOptions(void)
{
    optionsAction->setEnabled(false);
    Mapped::instance->setEnabled(false);
    mappedAction->setEnabled(false);    // kill menu entry...
    Options::instance->open();
    Options::instance->setEnabled(true);
    Options::instance->show();
}

void SwitchView::openMapped(void)
{
    if(!connected && !testing && !alwaysopen)
        warning();
    else if(!Mapped::instance->isVisible() && Mapped::instance->isEnabled()) {
        mappedAction->setEnabled(false);
        Mapped::instance->open();
    }
}

void SwitchView::mappedClosed(void)
{
    if(stopping)
        return;

    mappedAction->setEnabled(true);
}

void SwitchView::optionsClosed(void)
{
    if(stopping)
        return;

    optionsAction->setEnabled(true);
    Mapped::instance->setEnabled(true);
    if(!Mapped::instance->isVisible() && connected)
        mappedAction->setEnabled(true);
}

void SwitchView::action(QSystemTrayIcon::ActivationReason reason)
{
    switch(reason) {
    case QSystemTrayIcon::Trigger:
    case QSystemTrayIcon::DoubleClick:
        if(!connected && !testing) {
            warning();
            return;
        }
        mappedAction->setEnabled(false);
        if(Mapped::instance->isVisible())
            Mapped::instance->close();
        else if(Mapped::instance->isEnabled())
            Mapped::instance->open();
        return;
    default:
        return;
    }
}

